/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2013-2014, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "account.h"
#include "core/log.h"
#include "core/misc.h"
#include "functions/functions.h"



namespace Winix
{


namespace Fun
{

Account::Account()
{
	fun.url = L"account";
}



bool Account::ActivateAccount(User * puser, long code, bool use_ses_log)
{
	std::wstring * user_code_str = puser->aenv.GetValue(L"activation_code");

	if( user_code_str )
	{
		if( Tol(*user_code_str) == code )
		{
			if( db->ChangeUserStatus(puser->id, WINIX_ACCOUNT_READY) == WINIX_ERR_OK )
			{
				puser->aenv.Remove(L"activation_code");
				db->ChangeUserAdminEnv(puser->id, puser->aenv);
				puser->status = WINIX_ACCOUNT_READY;

				log << log2 << "Account: account: " << puser->name << " activated" << logend;

				if( use_ses_log )
					slog << loginfo << T(L"account_activated") << logend;

				return true;
			}
			else
			{
				log << log1 << "Account: account not activated -- database error" << logend;
			}
		}
		else
		{
			// !! IMPROVE ME if too many errors from the same IP address
			// add this ip to the banip list

			log << log2 << "Account: incorrect activation code" << logend;

			if( use_ses_log )
				slog << logerror << T(L"incorrect_activation_code") << logend;
		}
	}
	else
	{
		log << log1 << "Account: there is no activation_code value in an admin environment" << logend;

		if( use_ses_log )
			slog << loginfo << T(L"account_cannot_be_activated") << logend;
	}

return false;
}


bool Account::ActivateAccount(const std::wstring & login, long code, bool use_ses_log)
{
	bool result  = false;
	User * puser = system->users.GetUser(login);

	if( puser )
	{
		if( puser->status == WINIX_ACCOUNT_NOT_ACTIVATED )
		{
			result = ActivateAccount(puser, code, use_ses_log);
		}
		else
		{
			log << log2 << "Account: this account is already activated" << logend;

			if( use_ses_log )
				slog << loginfo << T(L"account_already_activated") << logend;
		}
	}
	else
	{
		log << log1 << "Account: there is no a user: " << login << logend;
	}

return result;
}



void Account::ActivateAccount()
{
	const std::wstring & login = cur->request->ParamValue(L"login");
	long code = Tol(cur->request->ParamValue(L"code"));

	if( !login.empty() )
	{
		ActivateAccount(login, code, true);
		system->RedirectToLastItem();
	}
}



void Account::MakePost()
{
}



void Account::MakeGet()
{
	if( cur->request->IsParam(L"activate") )
		ActivateAccount();
}




} // namespace


} // namespace Winix

